{-----------------------------------------------------------------------}
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to 2016 (LAZARUS),             }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{-----------------------------------------------------------------------}
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{-----------------------------------------------------------------------}
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{-----------------------------------------------------------------------}

unit u_objects_components;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  Classes, contnrs, Graphics, ExtCtrls, u_common_graph_type,
{$IFDEF FPC}
  LCLType, FPCanvas,
{$ELSE}
  Windows,
{$ENDIF}
  SysUtils;


type
  TPaintText = procedure(const AObject: TObject;
    const AString: string; const ALeft, ATop :Double;
    const AAngle : Integer = 0) of object;
  TPaintTextRect = procedure(const AObject: TObject;
    const ARect : TFloatRect;
    const AString: string; const ALeft, ATop :Double;
    const AAngle : Integer ) of object;
  TPaintPenSet = procedure(const AObject: TObject; const AColor:TColor;
    const PenStyle : {$IFDEF FPC}TFPPenStyle{$ENDIF};
    const PenWidth : Single ; const PenMode : {$IFDEF FPC}TFPPenMode{$ENDIF}) of object;
  TPaintBrushSet = procedure(const AObject: TObject;
    const AColor:TColor;const BrushStyle : {$IFDEF FPC}TFPBrushStyle{$ENDIF}) of object;
  TPaintColor = procedure(const AObject: TObject; const AColor : TColor ) of object;
//  TPaintFont  = procedure(const AObject: TObject; const FontName : String ; const FontSize : Integer ; const AColor : TColor ) of object;
  TPaintFont  = procedure(const AObject: TObject; const AFont : TFont ; const AFontSize : Single ) of object;
  TPaintCoord = procedure(const AObject: TObject; const AX, AY : Extended ) of object;
  TPaintRect = procedure(const AObject: TObject; const AX1, AY1, AX2, AY2 : Extended ) of object;
  TPaintRoundRect = procedure(const AObject: TObject; const AX1, AY1, AX2, AY2, W, H : Extended ) of object;
  TPaintArc = procedure(const AObject: TObject;
    const ARect : TFloatRect ; const ABeginArcXC, ABeginArcYC, AEndArcXC, AEndArcYC : Extended ) of object;
  TPerson = class
   private
    fLevel: integer;
    fCleFiche: integer;
    fChilds,
    fChildren: TObjectList;
    fClePere,
    fCleMere: int64;
    fMere,
    fPere: TPerson;

    fSexe: integer;

   protected

    fOrderTete: integer;
   public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Clear; virtual;
   published
    property KeyPerson: integer read fCleFiche write fCleFiche;
    property FatherKey: Int64 read fClePere write fClePere;
    property MotherKey: Int64 read fCleMere write fCleMere;
    property Father: TPerson read fPere write fPere;
    property Mother: TPerson read fMere write fMere;
    property Sexe: integer read fSexe write fSexe;
    property Childs: TObjectList read fChilds write fChilds;
    property Children: TObjectList read fChildren write fChildren;

    property Level: integer read fLevel write fLevel;
   end;

procedure AbortMessage ( const AText : String );

implementation

uses
  Controls,
  u_common_functions, Dialogs;

procedure AbortMessage ( const AText : String );
Begin
  ShowMessage(AText);
  Abort;
end;



{ TPerson }

procedure TPerson.Clear;
begin
  fSexe := -1;
  fCleFiche := -1;
  fLevel := 0;

  fCleMere := -1;
  fClePere := -1;

  fChilds.Clear;
  fChildren.Clear;
  fMere:=nil;
  fPere:=nil;
end;

constructor TPerson.Create;
begin
  fChilds := TObjectList.Create(True);
  fChildren := TObjectList.Create(True);
  Clear;
end;

destructor TPerson.Destroy;
begin
  fChilds.Free;

  inherited Destroy;
end;

end.

