{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to 2016 (LAZARUS),             }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_genealogy_context;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}
{$IFDEF WIN32}
  {$DEFINE WEBMAPS}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  windows,
{$ELSE}
  LCLType, LazFileUtils,
{$ENDIF}
  lazutf8classes,
  Classes,Graphics,Forms,
  u_common_const,
  fonctions_string,
  fonctions_net,
  fonctions_db,
  IniFiles, u_common_functions;

const CST_BACKUP = 'Backup';
      ANCESTROMANIA = 'Ancestromania';
      ANCESTROMANIA_DATABASE = ANCESTROMANIA+CST_EXTENSION_FIREBIRD;
      CST_INI_VLC = 'VLC';
      CST_INI_VLC_PLAYER = 'InternalPlayer';
      CST_INI_VLC_PLAYER_DEFAULT = {$IFDEF WINDOWS}False{$ELSE}True{$ENDIF};


type

  { TContextIni }

  TContextIni=class ( TComponent )
  private
    //La langue utilisée pour la saisie des données (dates, événements) (code)
    //elle est particulière à chaque dossier AL2009
    fLangue,
    fPath_Applis_Ancestro :string;
    //savoir si c'est le premier démarrage
    fFirstDemarrage,
    //--paramètres de l'index des noms
    fIndexFavecPat,
    fIndexFcolAuto,
    fOpenQuiSontIls : boolean;
    fValidationAutomatique:Boolean;
    //valeurs publiées en propriétés
    //pour savoir si il faut enregistrer les modifications
    fShouldSave:boolean;
    fNumVersion:integer;
    fFileNameReport:TStringlistUTF8;

    //--paramètres de la forme requêtes
    function  GetFirstFileNameReport(const aRelPathNameReport: string): string;
    function  GetZoom : Integer;
    procedure SetZoom ( const AValue : Integer );
    function  GetOngletMultimediaReseau : Boolean;
    procedure SetOngletMultimediaReseau ( const AValue : Boolean );
    function  GetTailleFenetre : Integer;
    procedure SetTailleFenetre ( const AValue : Integer );

    // -- Type de BDD ------------------------------------------------------
    function GetBDD_Ratio_Taille : SHORT;
    procedure SetBDD_Ratio_Taille ( const AValue : SHORT );
    // -- Base locale PARANCES des tables de référence des lieux ------------
    function GetUtilPARANCES :string;
    procedure SetUtilPARANCES ( const AValue : string );
    function GetPwUtilPARANCES  :string;
    procedure SetPwUtilPARANCES ( const AValue : string );
    function GetRolePARANCES  :string;
    procedure SetRolePARANCES ( const AValue : string );


    // -- Gedcom ----------------------------------------------------------
    function GetGedcom_Nom :string;
    procedure SetGedcom_Nom ( const AValue : string );
    function GetGedcom_Adr1 :string;
    procedure SetGedcom_Adr1 ( const AValue : string );
    function GetGedcom_Adr2 :string;
    procedure SetGedcom_Adr2 ( const AValue : string );
    function GetGedcom_Adr3 :string;
    procedure SetGedcom_Adr3 ( const AValue : string );
    function GetGedcom_Tel :string;
    procedure SetGedcom_Tel ( const AValue : string );
    function GetGedcom_Mail :string;
    procedure SetGedcom_Mail ( const AValue : string );
    function GetGedcom_Web :string;
    procedure SetGedcom_Web ( const AValue : string );

    function GetProductName :string;

    //La base de donnée
    function GetPathFileNameBdd :string;
    procedure SetPathFileNameBdd ( const AValue : string );

    //Les répertoires de l'application
    function GetPathQuiSontIls :string;
    procedure SetPathQuiSontIls ( const AValue : string );

    //Les répertoires utilisateurs
    function GetPathImportExport :string;
    procedure SetPathImportExport ( const AValue : string );
    function GetPathImages :string;
    procedure SetPathImages ( const AValue : string );
    function GetPathDocs :string;
    procedure SetPathDocs ( const AValue : string );
    function GetPathSauvegarde :string;
    procedure SetPathSauvegarde ( const AValue : string );

    //Le pays préférentiel lors de la recherche des lieux
    function GetPays : Integer;
    procedure SetPays ( const AValue : Integer );
    function GetPaysNom :string;
    procedure SetPaysNom ( const AValue : string );

    //Couleurs
    function GetColorHomme : Integer;
    procedure SetColorHomme ( const AValue : Integer );
    function GetColorFemme : Integer;
    procedure SetColorFemme ( const AValue : Integer );
    function GetColorLight : Integer;
    procedure SetColorLight ( const AValue : Integer );
    function GetColorMedium : Integer;
    procedure SetColorMedium ( const AValue : Integer );
    function GetColorDark : Integer;
    procedure SetColorDark ( const AValue : Integer );
    function GetColorTexteOnglets : Integer;
    procedure SetColorTexteOnglets ( const AValue : Integer );

    //JPeg
    function GetTauxCompressionJPeg : Integer;
    procedure SetTauxCompressionJPeg ( const AValue : Integer );

    //Nombre maximum de favoris par dossier
    function GetMaxFavoris : Integer;
    procedure SetMaxFavoris ( const AValue : Integer );

    // Arbre

    function GetArbreRecouvrement : Integer;
    procedure SetArbreRecouvrement ( const AValue : Integer );
    function GetArbreShowNbGeneration : Integer;
    procedure SetArbreShowNbGeneration ( const AValue : Integer );
    function GetArbreDescendance : boolean;
    procedure SetArbreDescendance ( const AValue : Boolean );
    function GetArbreInverse : boolean;
    procedure SetArbreInverse ( const AValue : Boolean );
    // Roue
    function GetRoueShowNbGeneration : Integer;
    procedure SetRoueShowNbGeneration ( const AValue : Integer );
    function GetRoueRecouvrement : Integer;
    procedure SetRoueRecouvrement ( const AValue : Integer );
    // Arbre
    function GetLiensRecouvrement : Integer;
    procedure SetLiensRecouvrement ( const AValue : Integer );

    //Mode de saisie
    function GetModeSaisieNom:TModeFormatText;
    procedure SetModeSaisieNom ( const AValue :TModeFormatText);
    function GetModeSaisiePrenom:TModeFormatText;
    procedure SetModeSaisiePrenom ( const AValue :TModeFormatText);

    //activation des aides à l'ouverture de la saisie rapide
    function GetAideSaisieRapideNom  : boolean;
    procedure SetAideSaisieRapideNom ( const AValue : Boolean );
    function GetAideSaisieRapidePrenom : boolean;
    procedure SetAideSaisieRapidePrenom ( const AValue : Boolean );
    function GetAideSaisieRapideProf : boolean;
    procedure SetAideSaisieRapideProf ( const AValue : Boolean );

    //activation de la création automatique de l'événement MARR
    function GetCreationMARRConjoint : boolean;
    procedure SetCreationMARRConjoint ( const AValue : Boolean );
    function GetCreationMARRParent : boolean;
    procedure SetCreationMARRParent ( const AValue : Boolean );


    function GetPhotos : boolean;
    procedure SetPhotos ( const AValue : Boolean );
    function GetMontrerRepere : boolean;
    procedure SetMontrerRepere ( const AValue : Boolean );
    function GetGarde : boolean;
    procedure SetGarde ( const AValue : Boolean );

    function GetCanSeeHint : boolean;
    procedure SetCanSeeHint ( const AValue : Boolean );
    function GetBackup : boolean;
    procedure SetBackup ( const AValue : Boolean );
    function GetAfficheCPdansListeEV : boolean;
    procedure SetAfficheCPdansListeEV ( const AValue : Boolean );
    function GetAfficheRegiondansListeEV : boolean;
    procedure SetAfficheRegiondansListeEV ( const AValue : Boolean );
    function GetShowNew : boolean;
    procedure SetShowNew ( const AValue : Boolean );
    function GetArbreReduit : Integer;
    procedure SetArbreReduit ( const AValue : Integer );
    function GetArbreHorizontal : boolean;
    procedure SetArbreHorizontal ( const AValue : Boolean );
    function GetVerifMAJ : boolean;
    procedure SetVerifMAJ ( const AValue : Boolean );
    function GetOuvreHistoire : boolean;
    procedure SetOuvreHistoire ( const AValue : Boolean );

    //les derniers éléments ouverts
    function GetLastNumDossier : integer;
    procedure SetLastNumDossier ( const AValue : integer );

    //Contrôles de cohérences
    function GetCoherenceActive : boolean;
    procedure SetCoherenceActive ( const AValue : Boolean );

    function GetAgeMaxiAuDecesHommes : Integer;
    procedure SetAgeMaxiAuDecesHommes ( const AValue : Integer );
    function GetAgeMaxiAuDecesFemmes : Integer;
    procedure SetAgeMaxiAuDecesFemmes ( const AValue : Integer );
    function GetAgeMiniMariageHommes : Integer;
    procedure SetAgeMiniMariageHommes ( const AValue : Integer );
    function GetAgeMiniMariageFemmes : Integer;
    procedure SetAgeMiniMariageFemmes ( const AValue : Integer );
    function GetAgeMaxiMariageHommes : Integer;
    procedure SetAgeMaxiMariageHommes ( const AValue : Integer );
    function GetAgeMaxiMariageFemmes : Integer;
    procedure SetAgeMaxiMariageFemmes ( const AValue : Integer );
    function GetAgeMiniNaissanceEnfantHommes : Integer;
    procedure SetAgeMiniNaissanceEnfantHommes ( const AValue : Integer );
    function GetAgeMiniNaissanceEnfantFemmes : Integer;
    procedure SetAgeMiniNaissanceEnfantFemmes ( const AValue : Integer );
    function GetAgeMaxiNaissanceEnfantHommes : Integer;
    procedure SetAgeMaxiNaissanceEnfantHommes ( const AValue : Integer );
    function GetAgeMaxiNaissanceEnfantFemmes : Integer;
    procedure SetAgeMaxiNaissanceEnfantFemmes ( const AValue : Integer );

    function GetNbJourEntre2NaissancesNormales : Integer;
    procedure SetNbJourEntre2NaissancesNormales ( const AValue : Integer );
    function GetNbJourEntre2NaissancesJumeaux : Integer;
    function GetEcartAgeEntreEpoux : Integer;
    procedure SetEcartAgeEntreEpoux ( const AValue : Integer );

    function GetCheckAgeMiniMariage : boolean;
    procedure SetCheckAgeMiniMariage ( const AValue : Boolean );
    function GetCheckAgeMaxiMariage : boolean;
    procedure SetCheckAgeMaxiMariage ( const AValue : Boolean );
    function GetCheckAgeMiniNaissanceEnfant : boolean;
    procedure SetCheckAgeMiniNaissanceEnfant ( const AValue : Boolean );
    function GetCheckAgeMaxiNaissanceEnfant : boolean;
    procedure SetCheckAgeMaxiNaissanceEnfant ( const AValue : Boolean );
    function GetCheckEsperanceVie : boolean;
    procedure SetCheckEsperanceVie ( const AValue : Boolean );
    function GetCheckEcartAgeEpoux : boolean;
    procedure SetCheckEcartAgeEpoux ( const AValue : Boolean );
    function GetCheckNbJourEntreNaissance : boolean;
    procedure SetCheckNbJourEntreNaissance ( const AValue : Boolean );

    // Divers

    function GetConSang : boolean;
    procedure SetConSang ( const AValue : Boolean );
    function GetNiveauConsang : integer;
    procedure SetNiveauConsang ( const AValue : Integer );
    function GetAvecLienGene : boolean;
    procedure SetAvecLienGene ( const AValue : Boolean );
    function GetCalculParenteAuto : boolean;
    procedure SetCalculParenteAuto ( const AValue : Boolean );
    function GetAstro : boolean;
    procedure SetAstro ( const AValue : Boolean );
    function GetSetAstro : integer;
    procedure SetSetAstro ( const AValue : integer );

    function GetPanelInfosHeight : Integer;
    procedure SetPanelInfosHeight ( const AValue : Integer );
    function GetPanelMediaWidth : Integer;
    procedure SetPanelMediaWidth ( const AValue : Integer );
    function GetSurnomAlaFin : boolean;
    procedure SetSurnomAlaFin ( const AValue : Boolean );
    function GetAfficherPatAssoc : boolean;
    procedure SetAfficherPatAssoc ( const AValue : Boolean );
    function GetCivilite : boolean;
    procedure SetCivilite ( const AValue : Boolean );
    function GetFormatSurnom : String;
    procedure SetFormatSurnom ( const AValue : string );
    function GetFormatSurnomF : String;
    procedure SetFormatSurnomF ( const AValue : string );
    function GetNomDefaut : String;
    procedure SetNomDefaut ( const AValue : string );
    function GetDistanceVillesVoisines : Integer;
    procedure SetDistanceVillesVoisines ( const AValue : Integer );
    function GetQuelSat : Integer;
    procedure SetQuelSat ( const AValue : Integer );

    function GetRepAutoWidth : boolean;
    procedure SetRepAutoWidth ( const AValue : Boolean );

    function GetHautEven : Integer;
    procedure SetHautEven ( const AValue : Integer );
    function GetCreateRepDefaut : boolean;
    procedure SetCreateRepDefaut ( const AValue : Boolean );

  public
    VersionExe:string;
    ImagesDansBase:boolean;

    property Zoom:integer read GetZoom write SetZoom;
    procedure WritePathNameBddIntoIni;


    constructor Create ( AOwner : TComponent ); override;
    destructor destroy; override;

    procedure LoadIni ( const AppName : String = '' );
    procedure SaveIni;
    procedure SaveOnlyIfShould;
    property FileNameReport:TStringlistUTF8 read fFileNameReport write fFileNameReport;

    property OngletMultimediaReseau:boolean read GetOngletMultimediaReseau write SetOngletMultimediaReseau;
    property TailleFenetre:integer read GetTailleFenetre write SetTailleFenetre;

    // -- Type de BDD ------------------------------------------------------
    property BDD_Ratio_Taille:SHORT read GetBDD_Ratio_Taille write SetBDD_Ratio_Taille;
    // -- Base locale PARANCES des tables de référence des lieux ------------
    property UtilPARANCES:string read GetUtilPARANCES write SetUtilPARANCES;
    property PwUtilPARANCES:string read GetPwUtilPARANCES write SetPwUtilPARANCES;
    property RolePARANCES:string read GetRolePARANCES write SetRolePARANCES;

    property OpenQuiSontIls:boolean read fOpenQuiSontIls write fOpenQuiSontIls;

    property NumVersion:integer read fNumVersion write fNumVersion;

    // -- Gedcom ----------------------------------------------------------
    property Gedcom_Nom:string read GetGedcom_Nom write SetGedcom_Nom;
    property Gedcom_Adr1:string read GetGedcom_Adr1 write SetGedcom_Adr1;
    property Gedcom_Adr2:string read GetGedcom_Adr2 write SetGedcom_Adr2;
    property Gedcom_Adr3:string read GetGedcom_Adr3 write SetGedcom_Adr3;
    property Gedcom_Tel:string read GetGedcom_Tel write SetGedcom_Tel;
    property Gedcom_Mail:string read GetGedcom_Mail write SetGedcom_Mail;
    property Gedcom_Web:string read GetGedcom_Web write SetGedcom_Web;

    property ProductName:string read GetProductName;

    //La base de donnée
    property PathFileNameBdd:string read GetPathFileNameBdd write SetPathFileNameBdd;

    //Les répertoires de l'application
    property Path_Applis_Ancestro:string read fPath_Applis_Ancestro write fPath_Applis_Ancestro;
    property PathQuiSontIls:string read GetPathQuiSontIls write SetPathQuiSontIls;

    //Les répertoires utilisateurs
    property PathImportExport:string read GetPathImportExport write SetPathImportExport;
    property PathImages:string read GetPathImages write SetPathImages;
    property PathDocs:string read GetPathDocs write SetPathDocs;
    property PathSauvegarde:string read GetPathSauvegarde write SetPathSauvegarde;

    //La langue utilisée pour la saisie des données (dates, ?v?nements) (code)
    //elle est particulière à chaque dossier AL2009
    property Langue:string read fLangue write fLangue;
    //Le pays préférentiel lors de la recherche des lieux
    property Pays:integer read GetPays write SetPays;
    property PaysNom:string read GetPaysNom write SetPaysNom;

    //Couleurs
    property ColorHomme:integer read GetColorHomme write SetColorHomme;
    property ColorFemme:integer read GetColorFemme write SetColorFemme;
    property ColorLight:integer read GetColorLight write SetColorLight;
    property ColorMedium:integer read GetColorMedium write SetColorMedium;
    property ColorDark:integer read GetColorDark write SetColorDark;
    property ColorTexteOnglets:integer read GetColorTexteOnglets write SetColorTexteOnglets;

    //JPeg
    property TauxCompressionJPeg:integer read GetTauxCompressionJPeg write SetTauxCompressionJPeg;

    //Nombre maximum de favoris par dossier
    property MaxFavoris:integer read GetMaxFavoris write SetMaxFavoris;

    // Arbre

    property ArbreRecouvrement:integer read GetArbreRecouvrement write SetArbreRecouvrement;
    property ArbreShowNbGeneration:integer read GetArbreShowNbGeneration write SetArbreShowNbGeneration;
    property ArbreDescendance:boolean read GetArbreDescendance write SetArbreDescendance;
    property ArbreInverse:boolean read GetArbreInverse write SetArbreInverse;
    // Roue
    property RoueShowNbGeneration:integer read GetRoueShowNbGeneration write SetRoueShowNbGeneration;
    property RoueRecouvrement:integer read GetRoueRecouvrement write SetRoueRecouvrement;
    // Arbre
    property LiensRecouvrement:integer read GetLiensRecouvrement write SetLiensRecouvrement;

    //Mode de saisie
    property ModeSaisieNom:TModeFormatText read GetModeSaisieNom write SetModeSaisieNom;
    property ModeSaisiePrenom:TModeFormatText read GetModeSaisiePrenom write SetModeSaisiePrenom;

    //activation des aides à l'ouverture de la saisie rapide
    property AideSaisieRapideNom:boolean read GetAideSaisieRapideNom write SetAideSaisieRapideNom;
    property AideSaisieRapidePrenom:boolean read GetAideSaisieRapidePrenom write SetAideSaisieRapidePrenom;
    property AideSaisieRapideProf:boolean read GetAideSaisieRapideProf write SetAideSaisieRapideProf;

    //activation de la création automatique de l'événement MARR
    property CreationMARRConjoint:boolean read GetCreationMARRConjoint write SetCreationMARRConjoint;
    property CreationMARRParent:boolean read GetCreationMARRParent write SetCreationMARRParent;


    property Photos:boolean read GetPhotos write SetPhotos;
    property MontrerRepere:boolean read GetMontrerRepere write SetMontrerRepere;
    property Garde:boolean read GetGarde write SetGarde;

    property CanSeeHint:boolean read GetCanSeeHint write SetCanSeeHint;
    property Backup:boolean read GetBackup write SetBackup;
    property AfficheCPdansListeEV:boolean read GetAfficheCPdansListeEV write SetAfficheCPdansListeEV;
    property AfficheRegiondansListeEV:boolean read GetAfficheRegiondansListeEV write SetAfficheRegiondansListeEV;
    property ShowNew:boolean read GetShowNew write SetShowNew;
    property ArbreReduit:integer read GetArbreReduit write SetArbreReduit;
    property ArbreHorizontal:boolean read GetArbreHorizontal write SetArbreHorizontal;
    property VerifMAJ:boolean read GetVerifMAJ write SetVerifMAJ;
    property OuvreHistoire:boolean read GetOuvreHistoire write SetOuvreHistoire;

    //les derniers ?l?ments ouverts
    property LastNumDossier:integer read GetLastNumDossier write SetLastNumDossier;

    //Contr?les de coh?rences
    property CoherenceActive:boolean read GetCoherenceActive write SetCoherenceActive;

    property AgeMaxiAuDecesHommes:integer read GetAgeMaxiAuDecesHommes write SetAgeMaxiAuDecesHommes;
    property AgeMaxiAuDecesFemmes:integer read GetAgeMaxiAuDecesFemmes write SetAgeMaxiAuDecesFemmes;
    property AgeMiniMariageHommes:integer read GetAgeMiniMariageHommes write SetAgeMiniMariageHommes;
    property AgeMiniMariageFemmes:integer read GetAgeMiniMariageFemmes write SetAgeMiniMariageFemmes;
    property AgeMaxiMariageHommes:integer read GetAgeMaxiMariageHommes write SetAgeMaxiMariageHommes;
    property AgeMaxiMariageFemmes:integer read GetAgeMaxiMariageFemmes write SetAgeMaxiMariageFemmes;
    property AgeMiniNaissanceEnfantHommes:integer read GetAgeMiniNaissanceEnfantHommes write SetAgeMiniNaissanceEnfantHommes;
    property AgeMiniNaissanceEnfantFemmes:integer read GetAgeMiniNaissanceEnfantFemmes write SetAgeMiniNaissanceEnfantFemmes;
    property AgeMaxiNaissanceEnfantHommes:integer read GetAgeMaxiNaissanceEnfantHommes write SetAgeMaxiNaissanceEnfantHommes;
    property AgeMaxiNaissanceEnfantFemmes:integer read GetAgeMaxiNaissanceEnfantFemmes write SetAgeMaxiNaissanceEnfantFemmes;

    property NbJourEntre2NaissancesNormales:integer read GetNbJourEntre2NaissancesNormales write SetNbJourEntre2NaissancesNormales;
    property NbJourEntre2NaissancesJumeaux:integer read GetNbJourEntre2NaissancesJumeaux;
    property EcartAgeEntreEpoux:integer read GetEcartAgeEntreEpoux write SetEcartAgeEntreEpoux;

    property CheckAgeMiniMariage:boolean read GetCheckAgeMiniMariage write SetCheckAgeMiniMariage;
    property CheckAgeMaxiMariage:boolean read GetCheckAgeMaxiMariage write SetCheckAgeMaxiMariage;
    property CheckAgeMiniNaissanceEnfant:boolean read GetCheckAgeMiniNaissanceEnfant write SetCheckAgeMiniNaissanceEnfant;
    property CheckAgeMaxiNaissanceEnfant:boolean read GetCheckAgeMaxiNaissanceEnfant write SetCheckAgeMaxiNaissanceEnfant;
    property CheckEsperanceVie:boolean read GetCheckEsperanceVie write SetCheckEsperanceVie;
    property CheckEcartAgeEpoux:boolean read GetCheckEcartAgeEpoux write SetCheckEcartAgeEpoux;
    property CheckNbJourEntreNaissance:boolean read GetCheckNbJourEntreNaissance write SetCheckNbJourEntreNaissance;

    //pour savoir si il faut enregistrer les modifications
    property ShouldSave:boolean read fShouldSave write fShouldSave;

    //savoir si c'est le premier d?marrage
    property FirstDemarrage:boolean read fFirstDemarrage;

    // Divers

    property ValidationAutomatique:boolean read fValidationAutomatique write fValidationAutomatique;
    property ConSang:boolean read GetConSang write SetConSang;
    property NiveauConsang:integer read GetNiveauConsang write SetNiveauConsang;
    property AvecLienGene:boolean read GetAvecLienGene write SetAvecLienGene;
    property CalculParenteAuto:boolean read GetCalculParenteAuto write SetCalculParenteAuto;
    property Astro:boolean read GetAstro write SetAstro;
    property PutAstro:integer read GetSetAstro write SetSetAstro;

    property PanelInfosHeight:integer read GetPanelInfosHeight write SetPanelInfosHeight;
    property PanelMediaWidth:integer read GetPanelMediaWidth write SetPanelMediaWidth;
    property SurnomAlaFin:boolean read GetSurnomAlaFin write SetSurnomAlaFin;
    property AfficherPatAssoc:boolean read GetAfficherPatAssoc write SetAfficherPatAssoc;
    property Civilite:boolean read GetCivilite write SetCivilite;
    property FormatSurnom:string read GetFormatSurnom write SetFormatSurnom;
    property FormatSurnomF:string read GetFormatSurnomF write SetFormatSurnomF;
    property NomDefaut:string read GetNomDefaut write SetNomDefaut;
    property DistanceVillesVoisines:integer read GetDistanceVillesVoisines write SetDistanceVillesVoisines;
    property QuelSat:integer read GetQuelSat write SetQuelSat;

    property RepAutoWidth:boolean read GetRepAutoWidth write SetRepAutoWidth;

    property HautEven:integer read GetHautEven write SetHautEven;
    property CreateRepDefaut:boolean read GetCreateRepDefaut write SetCreateRepDefaut;

    //--paramètres de l'index des noms
    property IndexFavecPat:boolean read fIndexFavecPat write fIndexFavecPat;
    property IndexFcolAuto:boolean read fIndexFcolAuto write fIndexFcolAuto;
  end;

const INI_SETTINGS = 'Settings';
      INI_PATH = 'Path';
      INI_GRAPHARBRE = 'GraphArbre';
      INI_GRAPHROUE = 'GraphRoue';
      INI_COHERENCE = 'Coherence';

var AIniFile : TInifile;
  _REP_PATH_REPORT:array[0.._MAX_REP_PATH] of string;
  gci_context:TContextIni=nil;
  fPath_Applis_Ancestro,fPathFileNameBdd:String;

implementation

uses
  {$IFDEF WIN32}
  JwaWinNT, JwaWinBase,
  {$ENDIF}
  fonctions_ini,sysutils,
  fonctions_system;

function IntToModeFormatText ( const k : Integer ): TModeFormatText;
Begin
  case k of
    1:Result:=mftUpper;
    2:Result:=mftFirstCharOfWordsIsMaj;
   else
     Result:=mftNone;
  end;
end;

function ModeFormatTextToInt ( const k : TModeFormatText ): Integer;
Begin
  case k of
    mftUpper:Result:=1;
    mftFirstCharOfWordsIsMaj:Result:=2;
    else
      Result:=0;
  end;
end;

{ TContextIni }

function TContextIni.GetFirstFileNameReport(const aRelPathNameReport:string):string;
var
  List:TStringlistUTF8;
begin
  result:='';
  List:=TStringlistUTF8.create;
  try
    GetDirContent(Path_Applis_Ancestro+_REL_PATH_REPORT+aRelPathNameReport,List,'*.rtm');
    if List.Count>0 then
      result:=List[0];
  finally
    List.Free;
  end;
end;

function TContextIni.GetZoom: Integer;
begin
  Result :=f_IniReadSectionInt(INI_SETTINGS,'Zoom',15);
end;

procedure TContextIni.SetZoom(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'Zoom',AValue);
end;

function TContextIni.GetOngletMultimediaReseau: Boolean;
begin
  Result :=f_IniReadSectionBol(INI_SETTINGS,'OngletMultimediaReseau',True);
end;

procedure TContextIni.SetOngletMultimediaReseau(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_SETTINGS,'OngletMultimediaReseau',AValue);
end;

function TContextIni.GetTailleFenetre: Integer;
begin
  Result :=f_IniReadSectionInt(INI_SETTINGS,'TailleFenetre',100);
end;

procedure TContextIni.SetTailleFenetre(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'TailleFenetre',AValue);
end;

function TContextIni.GetBDD_Ratio_Taille: SHORT;
begin
  Result :=f_IniReadSectionInt(INI_SETTINGS,'BDD_Ratio_Taille',20);
end;

procedure TContextIni.SetBDD_Ratio_Taille(const AValue: SHORT);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'BDD_Ratio_Taille',AValue);
end;

// -- Base locale PARANCES des tables de référence des lieux ------------
function TContextIni.GetUtilPARANCES: string;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'UtilPARANCES','SYSDBA');
end;

procedure TContextIni.SetUtilPARANCES(const AValue: string);
begin
  p_iniWriteSectionStr(INI_SETTINGS,'UtilPARANCES',AValue);
end;

function TContextIni.GetPwUtilPARANCES: string;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'PwUtilPARANCES','masterkey');
end;

procedure TContextIni.SetPwUtilPARANCES(const AValue: string);
begin
  p_iniWriteSectionStr(INI_SETTINGS,'PwUtilPARANCES',AValue);
end;

function TContextIni.GetRolePARANCES: string;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'RolePARANCES','');
end;

procedure TContextIni.SetRolePARANCES(const AValue: string);
begin
  f_iniReadSectionStr(INI_SETTINGS,'RolePARANCES',AValue);
end;


// -- Gedcom -----------------------------------------------------------

function TContextIni.GetGedcom_Nom: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Nom','');
end;

procedure TContextIni.SetGedcom_Nom(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Nom',AValue);
end;

function TContextIni.GetGedcom_Adr1: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Adr1','');
end;

procedure TContextIni.SetGedcom_Adr1(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Adr1',AValue);
end;

function TContextIni.GetGedcom_Adr2: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Adr2','');
end;

procedure TContextIni.SetGedcom_Adr2(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Adr2',AValue);
end;

function TContextIni.GetGedcom_Adr3: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Adr3','');
end;

procedure TContextIni.SetGedcom_Adr3(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Adr3',AValue);
end;

function TContextIni.GetGedcom_Tel: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Tel','');
end;

procedure TContextIni.SetGedcom_Tel(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Tel',AValue);
end;

function TContextIni.GetGedcom_Mail: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Mail','');
end;

procedure TContextIni.SetGedcom_Mail(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Mail',AValue);
end;

function TContextIni.GetGedcom_Web: string;
begin
  Result := f_iniReadSectionStr('Gedcom','Gedcom_Web','http://www.monsite.net');
end;

procedure TContextIni.SetGedcom_Web(const AValue: string);
begin
  p_iniWriteSectionStr('Gedcom','Gedcom_Web',AValue);
end;

function TContextIni.GetProductName: string;
{$IFDEF WIN32}
var Ver: POsVersionInfo;
begin
  new(ver);
  Ver^.dwOSVersionInfoSize := SizeOf(Ver);
  GetVersionEx(Ver);
  with Ver^ do begin
    case dwPlatformId of
      VER_PLATFORM_WIN32s: Result := '32s';
      VER_PLATFORM_WIN32_WINDOWS:
        begin
          dwBuildNumber := dwBuildNumber and $0000FFFF;
          if (dwMajorVersion > 4) or ((dwMajorVersion = 4) and
            (dwMinorVersion >= 10)) then
            Result := '98' else
            Result := '95';
        end;
      VER_PLATFORM_WIN32_NT: Result := 'NT';
    end;
  end;
end;
{$ELSE}
begin
  {$IFDEF LINUX}
  Result:='Linux';
  {$ELSE}
  {$IFDEF CARBON}
  Result:='Mac OSX Carbon';
  {$ELSE}
  {$IFDEF COCOA}
  Result:='iOS';
  {$ELSE}
  {$IFDEF BSD}
  Result:='BSD';
  {$ELSE}
  {$IFDEF UNIX}
  Result:='Unix';
  {$ELSE}
  Result:='Unknown';
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
end;
{$ENDIF}

//La base de donnée

function TContextIni.GetPathFileNameBdd: string;
begin
  Result := f_iniReadSectionStr(INI_PATH,'PathFileNameBdd','');
end;

procedure TContextIni.SetPathFileNameBdd(const AValue: string);
begin
  p_iniWriteSectionStr(INI_PATH,'PathFileNameBdd',AValue);
end;

function TContextIni.GetPathQuiSontIls: string;
begin
  Result :=ExcludeTrailingPathDelimiter(f_iniReadSectionStr(INI_PATH,'PathQuiSontIls',''));
end;

procedure TContextIni.SetPathQuiSontIls(const AValue: string);
begin
  p_iniWriteSectionStr(INI_PATH,'PathQuiSontIls',AValue);
end;

function TContextIni.GetPathImportExport: string;
begin
  Result := ExcludeTrailingPathDelimiter(f_iniReadSectionStr(INI_PATH,'PathImportExport',''));
end;

procedure TContextIni.SetPathImportExport(const AValue: string);
begin
  p_iniWriteSectionStr(INI_PATH,'PathImportExport',AValue);
end;

function TContextIni.GetPathImages: string;
begin
  Result := fPathBaseMedias;
end;

procedure TContextIni.SetPathImages(const AValue: string);
begin
  fPathBaseMedias:=AValue;
end;

function TContextIni.GetPathDocs: string;
begin
  Result := ExcludeTrailingPathDelimiter(f_iniReadSectionStr(INI_PATH,'PathDocuments',''));
end;

procedure TContextIni.SetPathDocs(const AValue: string);
begin
  f_iniReadSectionStr(INI_PATH,'PathDocuments',AValue);
end;

function TContextIni.GetPathSauvegarde: string;
begin
  Result := f_iniReadSectionStr(INI_PATH,'PathSauvegarde','');
end;

procedure TContextIni.SetPathSauvegarde(const AValue: string);
begin
  p_iniWriteSectionStr(INI_PATH,'PathSauvegarde',AValue);
end;

// Pays
function TContextIni.GetPays: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'Pays',1);
end;

procedure TContextIni.SetPays(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'Pays',AValue);
end;

function TContextIni.GetPaysNom: string;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'PaysNom','FRANCE');
end;

procedure TContextIni.SetPaysNom(const AValue: string);
begin
  p_iniWriteSectionStr(INI_SETTINGS,'PaysNom',AValue);
end;

//Couleurs
function TContextIni.GetColorHomme: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorHomme',$00FF8000);
end;

procedure TContextIni.SetColorHomme(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorHomme',AValue);
end;

function TContextIni.GetColorFemme: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorFemme',$008080FF);
end;

procedure TContextIni.SetColorFemme(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorFemme',AValue);

end;

function TContextIni.GetColorLight: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorLight',clWindow);
end;

procedure TContextIni.SetColorLight(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorLight',AValue);
end;

function TContextIni.GetColorMedium: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorMedium',clBtnFace);
end;

procedure TContextIni.SetColorMedium(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorMedium',AValue);
end;

function TContextIni.GetColorDark: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorDark',clDefault);
end;

procedure TContextIni.SetColorDark(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorDark',AValue);
end;

function TContextIni.GetColorTexteOnglets: Integer;
begin
  Result := f_IniReadSectionInt('Colors','ColorTexteOnglets',clWindowText);
end;

procedure TContextIni.SetColorTexteOnglets(const AValue: Integer);
begin
  p_iniWriteSectionInt('Colors','ColorTexteOnglets',AValue);
end;

function TContextIni.GetTauxCompressionJPeg: Integer;
begin
  Result :=f_IniReadSectionInt(INI_SETTINGS,'TauxCompressionJPeg',70);
end;

procedure TContextIni.SetTauxCompressionJPeg(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'TauxCompressionJPeg',AValue);
end;

//Nombre maximum de favoris par dossier
function TContextIni.GetMaxFavoris: Integer;
begin
  Result :=f_IniReadSectionInt(INI_SETTINGS,'MaxFavoris',20);
end;

procedure TContextIni.SetMaxFavoris(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'MaxFavoris',AValue);
end;

function TContextIni.GetArbreRecouvrement: Integer;
begin
  Result :=f_IniReadSectionInt(INI_GRAPHARBRE,'ArbreRecouvrement',2);
end;

procedure TContextIni.SetArbreRecouvrement(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_GRAPHARBRE,'ArbreRecouvrement',AValue);
end;

function TContextIni.GetArbreShowNbGeneration: Integer;
begin
  Result :=f_IniReadSectionInt(INI_GRAPHARBRE,'ArbreShowNbGeneration',12);
end;

procedure TContextIni.SetArbreShowNbGeneration(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_GRAPHARBRE,'ArbreShowNbGeneration',AValue);
end;

function TContextIni.GetArbreDescendance: boolean;
begin
  Result := f_IniReadSectionBol(INI_GRAPHARBRE,'ArbreDescendance',False);
end;

procedure TContextIni.SetArbreDescendance(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_GRAPHARBRE,'ArbreDescendance',AValue);
end;

function TContextIni.GetArbreInverse: boolean;
begin
  Result := f_IniReadSectionBol(INI_GRAPHARBRE,'ArbreInverse',False);
end;

procedure TContextIni.SetArbreInverse(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_GRAPHARBRE,'ArbreInverse',AValue);
end;

function TContextIni.GetRoueShowNbGeneration: Integer;
begin
  Result := f_IniReadSectionInt(INI_GRAPHROUE,'RoueShowNbGeneration',6);
end;

procedure TContextIni.SetRoueShowNbGeneration(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_GRAPHROUE,'RoueShowNbGeneration',AValue);
end;

function TContextIni.GetRoueRecouvrement: Integer;
begin
  Result := f_IniReadSectionInt(INI_GRAPHROUE,'RoueRecouvrement',2);
end;

procedure TContextIni.SetRoueRecouvrement(const AValue: Integer);
begin
  p_iniWriteSectionInt(INI_GRAPHROUE,'RoueRecouvrement',AValue);
end;

function TContextIni.GetLiensRecouvrement: Integer;
begin
  Result := f_IniReadSectionInt('GraphLiens','LiensRecouvrement',2);
end;

procedure TContextIni.SetLiensRecouvrement(const AValue: Integer);
begin
  p_iniWriteSectionInt('GraphLiens','LiensRecouvrement',AValue);
end;

function TContextIni.GetModeSaisieNom: TModeFormatText;
begin
  Result := IntToModeFormatText(f_IniReadSectionInt(INI_SETTINGS,'ModeSaisieNom',1));
end;

procedure TContextIni.SetModeSaisieNom(const AValue: TModeFormatText);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'ModeSaisieNom',ModeFormatTextToInt(AValue));
end;

function TContextIni.GetModeSaisiePrenom: TModeFormatText;
begin
  Result := IntToModeFormatText(f_IniReadSectionInt(INI_SETTINGS,'ModeSaisiePrenom',2));
end;

procedure TContextIni.SetModeSaisiePrenom(const AValue: TModeFormatText);
begin
  p_iniWriteSectionInt(INI_SETTINGS,'ModeSaisiePrenom',ModeFormatTextToInt(AValue));
end;

function TContextIni.GetAideSaisieRapideNom: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AideSaisieRapideNom',True);
end;

procedure TContextIni.SetAideSaisieRapideNom(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_SETTINGS,'AideSaisieRapideNom',AValue);
end;

function TContextIni.GetAideSaisieRapidePrenom: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AideSaisieRapidePrenom',True);
end;

procedure TContextIni.SetAideSaisieRapidePrenom(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_SETTINGS,'AideSaisieRapidePrenom',AValue);
end;

function TContextIni.GetAideSaisieRapideProf: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AideSaisieRapideProf',True);
end;

procedure TContextIni.SetAideSaisieRapideProf(const AValue: Boolean);
begin
  p_iniWriteSectionBol(INI_SETTINGS,'AideSaisieRapideProf',AValue);
end;

function TContextIni.GetCreationMARRConjoint: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'CreationMARRConjoint',False);
end;

procedure TContextIni.SetCreationMARRConjoint(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'CreationMARRConjoint',AValue);
end;

function TContextIni.GetCreationMARRParent: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'CreationMARRParent',False);
end;

procedure TContextIni.SetCreationMARRParent(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'CreationMARRParent',AValue);
end;

function TContextIni.GetPhotos: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'Photos',True);
end;

procedure TContextIni.SetPhotos(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'Photos',AValue);
end;

function TContextIni.GetMontrerRepere: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'MontrerRepere',True);
end;

procedure TContextIni.SetMontrerRepere(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'MontrerRepere',AValue);
end;

function TContextIni.GetGarde: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'Garde',False);
end;

procedure TContextIni.SetGarde(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'Garde',AValue);
end;

function TContextIni.GetCanSeeHint: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'CanSeeHint',True);
end;

procedure TContextIni.SetCanSeeHint(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'CanSeeHint',AValue);
end;

function TContextIni.GetBackup: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'Backup',False);
end;

procedure TContextIni.SetBackup(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'Backup',AValue);
end;

function TContextIni.GetAfficheCPdansListeEV: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AfficheCPdansListeEV',False);
end;

procedure TContextIni.SetAfficheCPdansListeEV(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'AfficheCPdansListeEV',AValue);
end;

function TContextIni.GetAfficheRegiondansListeEV: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AfficheRegiondansListeEV',False);
end;

procedure TContextIni.SetAfficheRegiondansListeEV(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'AfficheRegiondansListeEV',AValue);
end;

function TContextIni.GetShowNew: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'ShowNew',True);
end;

procedure TContextIni.SetShowNew(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'ShowNew',AValue);
end;

function TContextIni.GetArbreReduit: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'ArbreReduit',3);
end;

procedure TContextIni.SetArbreReduit(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'ArbreReduit',AValue);
end;

function TContextIni.GetArbreHorizontal: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'ArbreHorizontal',false);
end;

procedure TContextIni.SetArbreHorizontal(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'ArbreHorizontal',AValue);
end;

function TContextIni.GetVerifMAJ: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'VerifMAJ',False);
end;

procedure TContextIni.SetVerifMAJ(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'VerifMAJ',AValue);
end;

//gardé pour compatibilité avec dll Arbres et mise à jour
function TContextIni.GetOuvreHistoire: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'OuvreHistoire',True);
end;

procedure TContextIni.SetOuvreHistoire(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'OuvreHistoire',AValue);
end;

//les derniers éléments ouverts
function TContextIni.GetLastNumDossier: integer;
begin
  Result := f_IniReadSectionInt('Last','LastNumDossier',1);
end;

procedure TContextIni.SetLastNumDossier(const AValue: integer);
begin
  p_IniWriteSectionInt('Last','LastNumDossier',AValue);
end;

//Contrôles de cohérences
function TContextIni.GetCoherenceActive: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CoherenceActive',True);
end;

procedure TContextIni.SetCoherenceActive(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CoherenceActive',AValue);
end;

function TContextIni.GetAgeMaxiAuDecesHommes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiAuDecesHommes',111);
end;

procedure TContextIni.SetAgeMaxiAuDecesHommes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiAuDecesHommes',AValue);
end;

function TContextIni.GetAgeMaxiAuDecesFemmes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiAuDecesFemmes',115);
end;

procedure TContextIni.SetAgeMaxiAuDecesFemmes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiAuDecesFemmes',AValue);
end;

function TContextIni.GetAgeMiniMariageHommes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMiniMariageHommes',12);
end;

procedure TContextIni.SetAgeMiniMariageHommes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMiniMariageHommes',AValue);
end;

function TContextIni.GetAgeMiniMariageFemmes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMiniMariageFemmes',10);
end;

procedure TContextIni.SetAgeMiniMariageFemmes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMiniMariageFemmes',AValue);

end;

function TContextIni.GetAgeMaxiMariageHommes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiMariageHommes',85);
end;

procedure TContextIni.SetAgeMaxiMariageHommes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiMariageHommes',AValue);
end;

function TContextIni.GetAgeMaxiMariageFemmes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiMariageFemmes',85);
end;

procedure TContextIni.SetAgeMaxiMariageFemmes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiMariageFemmes',AValue);
end;

function TContextIni.GetAgeMiniNaissanceEnfantHommes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMiniNaissanceEnfantHommes',14);
end;

procedure TContextIni.SetAgeMiniNaissanceEnfantHommes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMiniNaissanceEnfantHommes',AValue);
end;

function TContextIni.GetAgeMiniNaissanceEnfantFemmes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMiniNaissanceEnfantFemmes',16);
end;

procedure TContextIni.SetAgeMiniNaissanceEnfantFemmes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMiniNaissanceEnfantFemmes',AValue);
end;

function TContextIni.GetAgeMaxiNaissanceEnfantHommes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiNaissanceEnfantHommes',65);
end;

procedure TContextIni.SetAgeMaxiNaissanceEnfantHommes(const AValue: Integer);
begin
 p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiNaissanceEnfantHommes',AValue);
end;

function TContextIni.GetAgeMaxiNaissanceEnfantFemmes: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'AgeMaxiNaissanceEnfantFemmes',97);
end;

procedure TContextIni.SetAgeMaxiNaissanceEnfantFemmes(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'AgeMaxiNaissanceEnfantFemmes',AValue);
end;

function TContextIni.GetNbJourEntre2NaissancesNormales: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'NbJourEntre2NaissancesNormales',300);
end;

procedure TContextIni.SetNbJourEntre2NaissancesNormales(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'NbJourEntre2NaissancesNormales',AValue);
end;

function TContextIni.GetNbJourEntre2NaissancesJumeaux: Integer;
begin
  Result := 2;
end;

function TContextIni.GetEcartAgeEntreEpoux: Integer;
begin
  Result := f_IniReadSectionInt(INI_COHERENCE,'EcartAgeEntreEpoux',40);
end;

procedure TContextIni.SetEcartAgeEntreEpoux(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_COHERENCE,'EcartAgeEntreEpoux',AValue);
end;

function TContextIni.GetCheckAgeMiniMariage: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckAgeMiniMariage',True);
end;

procedure TContextIni.SetCheckAgeMiniMariage(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckAgeMiniMariage',AValue);
end;

function TContextIni.GetCheckAgeMaxiMariage: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckAgeMaxiMariage',True);
end;

procedure TContextIni.SetCheckAgeMaxiMariage(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckAgeMaxiMariage',AValue);
end;

function TContextIni.GetCheckAgeMiniNaissanceEnfant: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckAgeMiniNaissanceEnfant',True);
end;

procedure TContextIni.SetCheckAgeMiniNaissanceEnfant(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckAgeMiniNaissanceEnfant',AValue);
end;

function TContextIni.GetCheckAgeMaxiNaissanceEnfant: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckAgeMaxiNaissanceEnfant',True);
end;

procedure TContextIni.SetCheckAgeMaxiNaissanceEnfant(const AValue: Boolean);
begin
   p_IniWriteSectionBol(INI_COHERENCE,'CheckAgeMaxiNaissanceEnfant',AValue);
end;

function TContextIni.GetCheckEsperanceVie: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckEsperanceVie',True);
end;

procedure TContextIni.SetCheckEsperanceVie(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckEsperanceVie',AValue);
end;

function TContextIni.GetCheckEcartAgeEpoux: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckEcartAgeEpoux',True);
end;

procedure TContextIni.SetCheckEcartAgeEpoux(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckEcartAgeEpoux',AValue);
end;

function TContextIni.GetCheckNbJourEntreNaissance: boolean;
begin
  Result := f_IniReadSectionBol(INI_COHERENCE,'CheckNbJourEntreNaissance',True);
end;

procedure TContextIni.SetCheckNbJourEntreNaissance(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_COHERENCE,'CheckNbJourEntreNaissance',AValue);
end;

function TContextIni.GetConSang: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'ConSang',True);
end;

procedure TContextIni.SetConSang(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'ConSang',AValue);
end;

function TContextIni.GetNiveauConsang: integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'NiveauConsang',4);
end;

procedure TContextIni.SetNiveauConsang(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'NiveauConsang',AValue);
end;

function TContextIni.GetAvecLienGene: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AvecLienGene',True);
end;

procedure TContextIni.SetAvecLienGene(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'AvecLienGene',AValue);
end;

function TContextIni.GetCalculParenteAuto: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'CalculParenteAuto',True);
end;

procedure TContextIni.SetCalculParenteAuto(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'CalculParenteAuto',AValue);
end;

function TContextIni.GetAstro: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'Astro',True);
end;

procedure TContextIni.SetAstro(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'Astro',AValue);
end;

function TContextIni.GetSetAstro: integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'SetAstro',3);
end;

procedure TContextIni.SetSetAstro(const AValue: integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'SetAstro',AValue);
end;

function TContextIni.GetPanelInfosHeight: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'PanelInfosHeight',127);
end;

procedure TContextIni.SetPanelInfosHeight(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'PanelInfosHeight',AValue);
end;

function TContextIni.GetPanelMediaWidth: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'PanelMediaWidth',350);
end;

procedure TContextIni.SetPanelMediaWidth(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'PanelMediaWidth',AValue);
end;

function TContextIni.GetSurnomAlaFin: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'SurnomAlaFin',false);
end;

procedure TContextIni.SetSurnomAlaFin(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'SurnomAlaFin',AValue);
end;

function TContextIni.GetAfficherPatAssoc: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'AfficherPatAssoc',True);
end;

procedure TContextIni.SetAfficherPatAssoc(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'AfficherPatAssoc',AValue);
end;

function TContextIni.GetCivilite: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'Civilite',True);
end;

procedure TContextIni.SetCivilite(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'Civilite',AValue);
end;

function TContextIni.GetFormatSurnom: String;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'FormatSurnom','');
end;

procedure TContextIni.SetFormatSurnom(const AValue: string);
begin
  p_IniWriteSectionStr(INI_SETTINGS,'FormatSurnom',AValue);
end;

function TContextIni.GetFormatSurnomF: String;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'FormatSurnomF','');
  if (Result='') and (FormatSurnom>'') then
    Result:=FormatSurnom+'e';
end;

procedure TContextIni.SetFormatSurnomF(const AValue: string);
begin
  p_IniWriteSectionStr(INI_SETTINGS,'FormatSurnomF',AValue);
end;

function TContextIni.GetNomDefaut: String;
begin
  Result := f_iniReadSectionStr(INI_SETTINGS,'NomDefaut','NOM DE L''INDIVIDU');
end;

procedure TContextIni.SetNomDefaut(const AValue: string);
begin
  p_IniWriteSectionStr(INI_SETTINGS,'NomDefaut',AValue);
end;

function TContextIni.GetDistanceVillesVoisines: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'DistanceVillesVoisines',15);
end;

procedure TContextIni.SetDistanceVillesVoisines(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'DistanceVillesVoisines',AValue);
end;

function TContextIni.GetQuelSat: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'QuelSat',  {$IFDEF WEBMAPS}0{$ELSE}3{$ENDIF});
end;

procedure TContextIni.SetQuelSat(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'QuelSat', AValue);
end;

function TContextIni.GetRepAutoWidth: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'RepAutoWidth',True);
end;

procedure TContextIni.SetRepAutoWidth(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'RepAutoWidth',AValue);
end;

function TContextIni.GetHautEven: Integer;
begin
  Result := f_IniReadSectionInt(INI_SETTINGS,'HautEven',525);
end;

procedure TContextIni.SetHautEven(const AValue: Integer);
begin
  p_IniWriteSectionInt(INI_SETTINGS,'HautEven',AValue);
end;

function TContextIni.GetCreateRepDefaut: boolean;
begin
  Result := f_IniReadSectionBol(INI_SETTINGS,'CreateRepDefaut',True);
end;

procedure TContextIni.SetCreateRepDefaut(const AValue: Boolean);
begin
  p_IniWriteSectionBol(INI_SETTINGS,'CreateRepDefaut',AValue);
end;

procedure TContextIni.WritePathNameBddIntoIni;
begin
  p_IniWriteSectionStr(INI_PATH,'PathFileNameBdd',fPathFileNameBdd);
end;

constructor TContextIni.Create ( AOwner : TComponent );
var
  n:integer;
begin
  inherited;
  ImagesDansBase:=false;
  fNumVersion:=StrToInt(NumVersionCourt(6,gr_ExeVersion))+StrToInt(NumVersionCourt(1, gr_ExeVersion))shl 16;
  VersionExe:=NumVersionCourt(2, gr_ExeVersion);
  fPath_MesDocuments:=IncludeTrailingPathDelimiter(GetDocDir)+Ancestromania+DirectorySeparator;
  {$IFDEF WINDOWS}
  fPath_Applis_Ancestro:=GetAppConfigDir(True)+DirectorySeparator;
  {$ELSE}
  fPath_Applis_Ancestro:=ExtractFileDir(Application.ExeName)+DirectorySeparator;
  {$ENDIF}

  // -- Divers ----------------------------
  fValidationAutomatique:=False;

  fFileNameReport:=TStringlistUTF8.create;
  for n:=0 to _MAX_REP_PATH do
    fFileNameReport.Add('');

end;

destructor TContextIni.destroy;
begin
  SaveOnlyIfShould;
  fFileNameReport.free;
  inherited;
end;

procedure TContextIni.LoadIni ( const AppName : String = '' );//serait mieux dans Initialization
var
  k:integer;
  FIniFile:TIniFile;

  procedure LoadStringList(Key:string;List:TStringlistUTF8);
  var
    n,p:integer;
  begin
    List.Clear;
    p:=f_IniReadSectionInt(Key,'Count',0);
    for n:=0 to p-1 do
      List.Add(f_iniReadSectionStr(Key,'Str'+inttostr(n),''));
  end;
begin
  fShouldSave:=False;

  f_GetMainMemIniFile( nil,nil, Self, False , AppName );
  try
    fFirstDemarrage:=GetPathFileNameBdd='';

      // BDD ------------------------------------------------------
    _user_name:=f_iniReadSectionStr(INI_SETTINGS,'IdUtilisateur',_user_name);
    _password:=f_iniReadSectionStr(INI_SETTINGS,'PwUtilisateur',_password);
    _role_name:=f_iniReadSectionStr(INI_SETTINGS,'RlUtilisateur','');
 

      //fichiers 'rtm' des rapports utilis?s
    for k:=0 to _MAX_REP_PATH do
    begin
      fFileNameReport[k]:=f_iniReadSectionStr('Reports','FileNameReport'+_REP_PATH_REPORT[k],'');

      if fFileNameReport[k]='' then
      begin
        fFileNameReport[k]:=GetFirstFileNameReport(_REP_PATH_REPORT[k]);
        if fFileNameReport[k]<>'' then
          fShouldSave:=True;
      end;
    end;

  finally
  end;

  //on vérifie que les répertoires existent
  if (not DirectoryExistsUTF8(PathImportExport)) then
    PathImportExport:=fPath_MesDocuments+_REL_PATH_IMPORT_EXPORT;
  if (not DirectoryExistsUTF8(PathDocs)) then
    PathDocs:=fPath_MesDocuments+_REL_PATH_DOC;

end;

procedure TContextIni.SaveIni;
var
  k:integer;
  FIniFile:TIniFile;

begin
  fShouldSave:=False;

  f_GetMemIniFile;
  try

      //fichiers 'rtm' des rapports utilis?s
    for k:=_MAX_REP_PATH downto 0 do
    begin//downto utilis? pour que la valeur de fFileNameReport[0] prime celles de k=1 ou 2
          //qui utilisent la m?me clef //AL
      if (fFileNameReport[k]<>'') then
        p_iniWriteSectionStr('Reports','FileNameReport'+_REP_PATH_REPORT[k],fFileNameReport[k]);
    end;

  finally
  end;
end;

procedure TContextIni.SaveOnlyIfShould;
begin
  if fShouldSave then SaveIni;
end;
end.

